# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-01-18 08:01
from __future__ import unicode_literals

import collectd_rest.models
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('collectd_rest', '0003_graphgranularity'),
    ]

    operations = [
        migrations.AlterField(
            model_name='graph',
            name='granularity',
            field=models.ForeignKey(default=collectd_rest.models.default_graph_granularity, on_delete=django.db.models.deletion.PROTECT, related_name='graphs', to='collectd_rest.GraphGranularity'),
        ),
    ]
